#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# %%
def ThermalConductivity(Temperature):
    """
    ========== DESCRIPTION ==========

    This function return the thermal conductivity of Nylon
    
    ========== Validity ==========

    4K < Temperature < 300K

    ========== FROM ==========
    
    E. D. Marquardt, J. P. Le, et R. Radebaugh, « Cryogenic Material Properties Database », p. 7, 2000.

    ========== INPUT ==========

    [Temperature]
        The temperature of the material in [K]

    ========== OUTPUT ==========

    [ThermalConductivity]
        The thermal conductivity in [W].[m]**(-1).[K]**(-1)

    ========== STATUS ==========

    Status : Checked

    """

    ################## MODULES ###############################################

    import numpy as np

    ################## CONDITIONS ############################################

    if Temperature <= 300 and Temperature >= 4:

        ################## INITIALISATION ####################################

        Coefficients = [-2.6135, 2.3239, -4.7586, 7.1602, -4.9155, 1.6324, -0.2507, 0.0131, 0]
        Sum = 0
        ################## IF CONDITION TRUE #####################

        for i in range(len(Coefficients)):
            Sum = Sum + Coefficients[i] * np.log10(Temperature) ** i

        return 10 ** Sum

        ################## SINON NAN #########################################

    else:

        print('Warning: The thermal conductivity of Nylon is not defined for T = ' + str(Temperature) + ' K')
        return np.nan


# %%
def LinearThermalExpansion(Temperature):
    """
    ========== DESCRIPTION ==========

    This function return the linear thermal expansion of Nylon
    
    ========== Validity ==========

    4K < Temperature < 300K

    ========== FROM ==========
    
    E. D. Marquardt, J. P. Le, et R. Radebaugh, « Cryogenic Material Properties Database », p. 7, 2000.

    ========== INPUT ==========

    [Temperature]
        The temperature of the material in [K]

    ========== OUTPUT ==========

    [LinearThermalExpansion]
        The linear thermal expansion in [%]

    ========== STATUS ==========

    Status : Checked

    """

    ################## MODULES ###############################################

    ################## CONDITIONS ############################################

    if Temperature <= 300 and Temperature >= 4:

        ################## INITIALISATION ####################################

        Coefficients = [-1.389e3, -1.561e-1, 2.988e-2, -7.948e-5, 1.181e-7]
        Sum = 0

        ################## IF CONDITION TRUE #####################

        for i in range(len(Coefficients)):
            Sum = Sum + Coefficients[i] * Temperature ** i

        return Sum * 1e-5

        ################## SINON NAN #########################################

    else:

        print('Warning: The linear thermal expansion of Nylon is not defined for T = ' + str(Temperature) + ' K')
        return np.nan



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# %%
def command_reset(address, channel):
    """
    ========== DESCRIPTION ==========

    This function can reset the parameters of a given channel on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    command = 'smu' + channel + '.reset()'

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_current_measure_autorange(address, channel, autorange):
    """
    ========== DESCRIPTION ==========

    This function can enable the autorange of the current measurement
    of a given channel on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <autorange>
        -- string --
        The autorange status ('ON' or 'OFF')
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    command = 'smu' + channel + '.measure.autorangei=smu' + channel + '.AUTORANGE_' + autorange

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_voltage_measure_autorange(address, channel, autorange):
    """
    ========== DESCRIPTION ==========

    This function can enable the autorange of the voltage measurement
    of a given channel on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <autorange>
        -- string --
        The autorange status ('ON' or 'OFF')
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    command = 'smu' + channel + '.measure.autorangev=smu' + channel + '.AUTORANGE_' + autorange

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_current_measure_range(address, channel, range_value):
    """
    ========== DESCRIPTION ==========

    This function can setup the range of the current measurement
    of a given channel on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <range_value>
        -- int --
        The range value (0 = 100 nA,
                         1 = 1 uA,
                         2 = 10 uA,
                         3 = 100 uA,
                         4 = 1 mA,
                         5 = 10 mA,
                         6 = 100 mA,
                         7 = 1 A,
                         8 = 3 A)
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    intensity_range = ['100e-9', '1e-6', '10e-6', '100e-6', '1e-3', '10e-3', '100e-3', '1', '3']

    range_value = intensity_range[range_value]

    command = 'smu' + channel + '.measure.rangei=' + range_value

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_voltage_measure_range(address, channel, range_value):
    """
    ========== DESCRIPTION ==========

    This function can setup the range of the voltage measurement
    of a given channel on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <range_value>
        -- int --
        The range value (0 = 100 mV,
                         1 = 1 V,
                         2 = 6 V,
                         3 = 40 V)
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    intensity_range = ['100e-3', '1', '6', '40']

    range_value = intensity_range[range_value]

    command = 'smu' + channel + '.measure.rangev=' + range_value

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def query_current(address, channel):
    """
    ========== DESCRIPTION ==========

    This function can return the measured current of a given channel on 
    Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')
        
    ========== OUTPUT ==========

    <current>
        -- float --
        The measured current
        [A]

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    command = 'print(smu' + channel + '.measure.i())'
    answer = instru.query(command)

    ################## FUNCTION ###############################################

    answer = float(answer)

    return answer


# %%
def query_voltage(address, channel):
    """
    ========== DESCRIPTION ==========

    This function can return the measured voltage of a given channel on 
    Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')
        
    ========== OUTPUT ==========

    <voltage>
        -- float --
        The measured voltage
        [V]

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    command = 'print(smu' + channel + '.measure.v())'
    answer = instru.query(command)

    ################## FUNCTION ###############################################

    answer = float(answer)

    return answer


# %%
def query_resistance(address, channel):
    """
    ========== DESCRIPTION ==========

    This function can return the measured resistance of a given channel on 
    Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')
        
    ========== OUTPUT ==========

    <resistance>
        -- float --
        The measured resistance
        [Ohm]

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    command = 'print(smu' + channel + '.measure.r())'
    answer = instru.query(command)

    ################## FUNCTION ###############################################

    answer = float(answer)

    return answer


# %%
def query_power(address, channel):
    """
    ========== DESCRIPTION ==========

    This function can return the measured power of a given channel on 
    Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')
        
    ========== OUTPUT ==========

    <voltage>
        -- float --
        The measured voltage
        [V]

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    command = 'print(smu' + channel + '.measure.p())'
    answer = instru.query(command)

    ################## FUNCTION ###############################################

    answer = float(answer)

    return answer


# %%
def command_current_source_autorange(address, channel, autorange):
    """
    ========== DESCRIPTION ==========

    This function can enable the autorange of the current source
    of a given channel on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <autorange>
        -- string --
        The autorange status ('ON' or 'OFF')
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    command = 'smu' + channel + '.source.autorangei=smu' + channel + '.AUTORANGE_' + autorange

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_voltage_source_autorange(address, channel, autorange):
    """
    ========== DESCRIPTION ==========

    This function can enable the autorange of the voltage source
    of a given channel on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <autorange>
        -- string --
        The autorange status ('ON' or 'OFF')
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    command = 'smu' + channel + '.source.autorangev=smu' + channel + '.AUTORANGE_' + autorange

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_source(address, channel, source):
    """
    ========== DESCRIPTION ==========

    This function can define the source type of a given channel on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <source>
        -- string --
        The source type ('VOLTS' or 'AMPS')
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    command = 'smu' + channel + '.source.func=smu' + channel + '.OUTPUT_' + source

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_source_current_value(address, channel, value):
    """
    ========== DESCRIPTION ==========

    This function can define the source current value of a given channel 
    on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <value>
        -- float --
        The current value
        [A]
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    command = 'smu' + channel + '.source.leveli=' + str(value)

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_source_voltage_value(address, channel, value):
    """
    ========== DESCRIPTION ==========

    This function can define the source voltage value of a given channel 
    on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <value>
        -- float --
        The voltage value
        [V]
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    command = 'smu' + channel + '.source.levelv=' + str(value)

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_source_current_limit(address, channel, limit):
    """
    ========== DESCRIPTION ==========

    This function can define the source current value limit of a given channel 
    on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <limit>
        -- float --
        The current limit value
        [A]
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    command = 'smu' + channel + '.source.limiti=' + str(limit)

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_source_voltage_limit(address, channel, limit):
    """
    ========== DESCRIPTION ==========

    This function can define the source voltage limit value of a given channel 
    on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <limit>
        -- float --
        The voltage limit value
        [V]
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    command = 'smu' + channel + '.source.limitv=' + str(limit)

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_source_power_limit(address, channel, limit):
    """
    ========== DESCRIPTION ==========

    This function can define the source power limit value of a given channel 
    on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <limit>
        -- float --
        The power limit value
        [W]
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    command = 'smu' + channel + '.source.limitp=' + str(limit)

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_source_status(address, channel, status):
    """
    ========== DESCRIPTION ==========

    This function can define the source power limit value of a given channel 
    on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <status>
        -- string --
        The status of the output source ('ON' or 'OFF')
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    command = 'smu' + channel + '.source.output=smu' + channel + '.OUTPUT_' + status

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_current_source_range(address, channel, range_value):
    """
    ========== DESCRIPTION ==========

    This function can setup the range of the current source
    of a given channel on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <range_value>
        -- int --
        The range value (0 = 100 nA,
                         1 = 1 uA,
                         2 = 10 uA,
                         3 = 100 uA,
                         4 = 1 mA,
                         5 = 10 mA,
                         6 = 100 mA,
                         7 = 1 A,
                         8 = 3 A)
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    intensity_range = ['100e-9', '1e-6', '10e-6', '100e-6', '1e-3', '10e-3', '100e-3', '1', '3']

    range_value = intensity_range[range_value]

    command = 'smu' + channel + '.source.rangei=' + range_value

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_voltage_source_range(address, channel, range_value):
    """
    ========== DESCRIPTION ==========

    This function can setup the range of the voltage source
    of a given channel on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <range_value>
        -- int --
        The range value (0 = 100 mV,
                         1 = 1 V,
                         2 = 6 V,
                         3 = 40 V)
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    intensity_range = ['100e-3', '1', '6', '40']

    range_value = intensity_range[range_value]

    command = 'smu' + channel + '.source.rangev=' + range_value

    ################## FUNCTION ###############################################

    instru.write(command)

    return


# %%
def command_sense_type(address, channel, sense_type):
    """
    ========== DESCRIPTION ==========

    This function can setup the sense type
    of a given channel on Keithley 2602

    ========== FROM ==========

    Manual of Keithley 2602

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- string --
        The channel of the instrument ('a' or 'b')

    <sense_type>
        -- string --
        The sense type ('LOCAL' or 'REMOTE')
     
    ========== OUTPUT ==========

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    command = 'smu' + channel + '.sense=smu' + channel + '.SENSE_' + sense_type

    ################## FUNCTION ###############################################

    instru.write(command)

    return



# -*- coding: utf-8 -*-
"""
Created on Tue Mar 22 19:33:02 2022

@author: Utilisateur
"""

import pandas as pd
import pyvisa

Fuite = pd.DataFrame({'datetime': [], 'leak_rate': [], 'pressure': []})

rm = pyvisa.ResourceManager()
rm.list_resources()
asm142 = rm.open_resource('COM8')

asm142.timeout = 5000
asm142.write('!VE')

test = asm142.read()
# leak_rate = float(test[21:29])
# pressure = float(test[32:40])
# timedate = datetime.now()
# data = {'datetime':timedate ,'leak_rate':leak_rate,'pressure':pressure}
# Fuite = Fuite.append(pd.DataFrame(data))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# %%
def thermal_expansion(temperature):
    """
    ========== DESCRIPTION ==========

    This function return the thermal expansion coefficient of HoY (Holmium-Yttrium)

    ========== VALIDITY ==========

    <temperature> : [4.2 -> 335]

    ========== FROM ==========

    Internal report - contact valentin.sauvage@outlook.com 

    ========== INPUT ==========

    <temperature>
        -- float --
        The temperature of the material
        [K]

    ========== OUTPUT ==========

    <thermal_expansion>
        -- float --
        The thermal expansion coefficient of Holmium-Yttrium
        [Ø]

    ========== STATUS ==========

    Status : Checked

    """

    ################## CONDITIONS #############################################

    assert temperature <= 335 and temperature >= 4.2, 'The function ' \
                                                      ' HoY.thermal_expansion is not defined for ' \
                                                      'T = ' + str(temperature) + ' K'

    ################## FUNCTION ###############################################

    expansion = 1.568530e-7 * temperature ** 4 - 1.595905e-4 * temperature ** 3 + 6.090793e-2 * temperature ** 2 - 1.757427 * temperature - 1.859420e3

    return expansion * 1e-6



# -*- coding: utf-8 -*- 
def PhononThermalConductivity(Temperature, Parameter):
    """
    ========== DESCRIPTION ==========
    
    This function can return the thermal conductivity of a material by phonons

    ========== VALIDITY ==========
    
    Temperature < DebyeTemperature/10

    ========== FROM ==========
    
    POBELL (1995) - Matter and Methods at Low Temperatures - P. 62
    

    ========== INPUT ==========
    
    [Temperature]
        The temperature of the material in [K]
        
    [Parameter]
        A coefficient in [W].[m]**(-1).[K]**(-1)
 
    ========== OUTPUT ==========
    
    [PhononThermalConductivity]
        The thermal conductivity (phonon contribution) in [W].[m]**(-1).[K]**(-1)
        
    ========== STATUS ==========     
    
    Status : TBC
    
    ========= IMPROVEMENT ========== 
    

    """

    ################## INITIALISATION ####################################

    return Parameter * Temperature ** 3


# %%
def ElectronSuperconductorThermalConductivity(Temperature, CriticalTemperature, Parameter):
    """
    ========== DESCRIPTION ==========
    
    This function can return the thermal conductivity of a material by electrons in superconducting state

    ========== VALIDITY ==========
    
    Temperature < CriticalTemperature

    ========== FROM ==========
    
    POBELL (1995) - Matter and Methods at Low Temperatures - P. 63
    

    ========== INPUT ==========
    
    [Temperature]
        The temperature of the material in [K]
        
    [CriticalTemperature]
        The critical temperature of the material in [K]
        
    [Parameter]
        A coefficient in [W].[m]**(-1).[K]**(-1)
 
    ========== OUTPUT ==========
    
    [ElectronSuperconductorThermalConductivity]
        The thermal conductivity (electron contribution - superconducting state) in [W].[m]**(-1).[K]**(-1)
        
    ========== STATUS ==========     
    
    Status : TBC
    
    ========= IMPROVEMENT ========== 
    

    """

    ################## MODULES ###############################################

    import numpy as np

    ################## INITIALISATION ####################################

    return Parameter * Temperature * np.exp(-1.76 * CriticalTemperature / Temperature)


# %%
def ElectronNormalThermalConductivity(Temperature, Parameter):
    """
    ========== DESCRIPTION ==========
    
    This function can return the thermal conductivity of a material by electrons in normal state

    ========== VALIDITY ==========
    
    Temperature > CriticalTemperature
    Temperature < 10 K

    ========== FROM ==========
    
    POBELL (1995) - Matter and Methods at Low Temperatures - P. 62
    

    ========== INPUT ==========
    
    [Temperature]
        The temperature of the material in [K]
        
    [Parameter]
        A coefficient in [W].[m]**(-1).[K]**(-1)
 
    ========== OUTPUT ==========
    
    [ElectronNormalThermalConductivity]
        The thermal conductivity (electron contribution - normal state) in [W].[m]**(-1).[K]**(-1)
        
    ========== STATUS ==========     
    
    Status : TBC
    
    ========= IMPROVEMENT ========== 
    

    """

    ################## MODULES ###############################################

    ################## INITIALISATION ####################################

    return Parameter * Temperature


# %%
def ElectronThermalConductivity(Temperature, CriticalTemperature, Parameter):
    """
    ========== DESCRIPTION ==========
    
    This function can return the thermal conductivity of a material by electrons

    ========== VALIDITY ==========
    
    Temperature < 10 K

    ========== FROM ==========
    
    POBELL (1995) - Matter and Methods at Low Temperatures - P. 62
    

    ========== INPUT ==========
    
    [Temperature]
        The temperature of the material in [K]
        
    [Parameter]
        A coefficient in [W].[m]**(-1).[K]**(-1)
        
    [CriticalTemperature]
        The critical temperature of the material in [K]
        
    ========== OUTPUT ==========
    
    [ElectronThermalConductivity]
        The thermal conductivity (electron contribution) in [W].[m]**(-1).[K]**(-1)
        
    ========== STATUS ==========     
    
    Status : TBC
    
    ========= IMPROVEMENT ========== 
    

    """

    ################## MODULES ###############################################

    from cryopy.Function import HeatTransfert
    import numpy as np

    ################## INITIALISATION ####################################

    ParameterSuperconductor = Parameter
    ParameterNormal = Parameter * np.exp(-1.76)

    if Temperature <= CriticalTemperature:
        return HeatTransfert.ElectronSuperconductorThermalConductivity(Temperature, CriticalTemperature,
                                                                       ParameterSuperconductor)
    else:
        return HeatTransfert.ElectronNormalThermalConductivity(Temperature, ParameterNormal)


# %%
def ThermalConductivity(Temperature, CriticalTemperature, ParameterElectron, ParameterPhonon):
    """
    ========== DESCRIPTION ==========
    
    This function can return the thermal conductivity of a material

    ========== VALIDITY ==========
    
    Temperature < 10 K

    ========== FROM ==========
    
    POBELL (1995) - Matter and Methods at Low Temperatures - P. 62
    

    ========== INPUT ==========
    
    [Temperature]
        The temperature of the material in [K]
        
    [CriticalTemperature]
        The critical temperature of the material in [K]
        
    [ParameterElectron]
        A coefficient in [W].[m]**(-1).[K]**(-1)

    [ParameterPhonon]
        A coefficient in [W].[m]**(-1).[K]**(-1)
        
    ========== OUTPUT ==========
    
    [ThermalConductivity]
        The thermal conductivity in [W].[m]**(-1).[K]**(-1)
        
    ========== STATUS ==========     
    
    Status : TBC
    
    ========= IMPROVEMENT ========== 
    

    """

    ################## MODULES ###############################################

    from cryopy.Function import HeatTransfert

    ################## INITIALISATION ####################################

    return HeatTransfert.ElectronThermalConductivity(Temperature, CriticalTemperature,
                                                     ParameterElectron) + HeatTransfert.PhononThermalConductivity(
        Temperature, ParameterPhonon)



# -*- coding: utf-8 -*-

# %%
def query_identification(address):
    """
    ========== DESCRIPTION ==========

    This function can return the identification of the LakeShore Model 336
    Cryogenic Temperature Controller

    ========== FROM ==========

    Manual of Lakeshore 336 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')

    ========== OUTPUT ==========

    <manufacturer>
        -- string --
        Should be "LSCI"

    <model>
        -- string --
        Should be "MODEL336"

    <instrument_serial>
        -- string --
        Depend on your instrument

    <option_serial>
        -- string --
        Depend on your instrument

    <firmware_version>
        -- string --
        Depend on your instrument

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    from cryopy import cryopy.Instrument.Lakeshore.Lakeshore336
    manufacturer,model,instrument_serial,option_serial,firmware_version = Lakeshore336.query_identification('GPIB0::15::INSTR')

    """

    ################## MODULES ###############################################

    import pyvisa

    ################## INITIALISATION ########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    answer = instru.query('*IDN?')

    manufacturer = answer[0:4]
    model = answer[5:13]
    instrument_serial = answer[14:21]
    option_serial = answer[22:29]
    firmware_version = answer[30:33]

    command_remote_interface_mode(address, 0)

    return manufacturer, model, instrument_serial, option_serial, firmware_version


# %%
def query_temperature(address):
    """
    ========== DESCRIPTION ==========

    This function can return the measured temperatures of the four channels
    A, B, C and D of the LakeShore Model 336 Cryogenic Temperature Controller

    ========== FROM ==========

    Manual of Lakeshore 336 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')

    ========== OUTPUT ==========

    <temperature_a>
        -- float --
        The measured temperature of the channel A
        [K]

    <temperature_b>
        -- float --
        The measured temperature of the channel B
        [K]

    <temperature_c>
        -- float --
        The measured temperature of the channel C
        [K]

    <temperature_d>
        -- float --
        The measured temperature of the channel D
        [K]

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    from cryopy import cryopy.Instrument.Lakeshore.Lakeshore336
    
    temperature_a,temperature_b,temperature_c,temperature_d = Lakeshore336.temperature_query('GPIB0::15::INSTR')


    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    answer = instru.query('KRDG? 0')

    temperature_a = float(answer[0:8])
    temperature_b = float(answer[9:17])
    temperature_c = float(answer[18:26])
    temperature_d = float(answer[27:35])

    command_remote_interface_mode(address, 0)

    return temperature_a, temperature_b, temperature_c, temperature_d


# %%
def query_input_reading_status(address):
    """
    ========== DESCRIPTION ==========

    This function can return the input reading status

    ========== FROM ==========

    Manual of Lakeshore 336 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')

    ========== OUTPUT ==========

    <status_indicator>
        -- string --
        The status of input reading
        []

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    from cryopy import cryopy.Instrument.Lakeshore.Lakeshore336
    
    input_reading_status = Lakeshore336.input_reading_status_query('GPIB0::15::INSTR')

    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    answer = instru.query('RDGST?')

    ################## FUNCTION ###############################################

    command_remote_interface_mode(address, 0)

    if int(answer[0:3]) == 0:
        return 'valid reading'

    if int(answer[0:3]) == 1:
        return 'invalid reading'

    if int(answer[0:3]) == 16:
        return 'temp underrange'

    if int(answer[0:3]) == 32:
        return 'temp overrange'

    if int(answer[0:3]) == 64:
        return 'sensor unit zero'

    if int(answer[0:3]) == 128:
        return 'sensor unit overrange'


# %%
def query_pid_parameters(address, output):
    """
    ========== DESCRIPTION ==========

    This function can return the PID parameters

    ========== FROM ==========

    Manual of Lakeshore 336 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')

    <output>
        -- int --
        The channel output (1 = output 1,
                            2 = output 2)
        []

    ========== OUTPUT ==========

    <p>
        -- float --
        The P parameter
        []

    <i>
        -- float --
        The I parameter
        []
        
    <d>
        -- float --
        The D parameter
        []
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========

    from cryopy import cryopy.Instrument.Lakeshore.Lakeshore336
    
    p,i,d= Lakeshore336.pid_parameters_query('GPIB0::15::INSTR')

    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    output = str(output)
    answer = instru.query('PID? ' + output)

    ################## FUNCTION ###############################################

    p = int(answer[0:5])
    i = int(answer[8:13])
    d = int(answer[16:20])

    command_remote_interface_mode(address, 0)

    return p, i, d


# %%
def command_pid_parameters(address, p, i, d, output):
    """
    ========== DESCRIPTION ==========

    This function can setup the PID parameters on specific output

    ========== FROM ==========

    Manual of Lakeshore 336 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')
        
    <p>
        -- float --
        The P parameter
        []

    <i>
        -- float --
        The I parameter
        []
        
    <d>
        -- float --
        The D parameter
        []
        
    <output>
        -- int --
        The channel output (1 = output 1,
                            2 = output 2)
        []

    ========== OUTPUT ==========
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    from cryopy import cryopy.Instrument.Lakeshore.Lakeshore336
    
    Lakeshore336.pid_parameters_command('GPIB0::15::INSTR',P,I,D)

    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    p = str(p)
    i = str(i)
    d = str(d)
    output = str(output)

    answer = 'PID ' + output + ',' + p + ',' + i + ',' + d

    ################## FUNCTION ###############################################

    instru.write(answer)

    command_remote_interface_mode(address, 0)

    return


# %%
def command_remote_interface_mode(address, mode):
    """
    ========== DESCRIPTION ==========

    This function can setup the remote interface mode command

    ========== FROM ==========

    Manual of Lakeshore 336 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')
        
    <mode>
        -- int --
        The mode of the interface command (0 = local, 
                                           1 = remote, 
                                           2 = remote with local lockout)
        []

    ========== OUTPUT ==========
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    "I want to setup the interface mode to remote"
    
    from cryopy import cryopy.Instrument.Lakeshore.Lakeshore336
    
    Lakeshore336.remote_interface_mode_command('GPIB0::15::INSTR',1)

    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)
    mode = str(mode)

    ################## FUNCTION ###############################################

    instru.write('MODE ' + mode)

    return


# %%
def command_autotune(address, output, mode):
    """
    ========== DESCRIPTION ==========

    This function can setup the autotune command on specific output

    ========== FROM ==========

    Manual of Lakeshore 336 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')
        
    <output>
        -- int --
        The channel output (1 = output 1,
                            2 = output 2,
                            3 = output 3,
                            4 = output 4)
        []
        
    <mode>
        -- int --
        The mode of the interface command (0 = P only,
                                           1 = P and I, 
                                           2 = P, I and D)
        []
        
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    "I want to setup the autotune with P and I parameters on output 2"
    
    from cryopy import cryopy.Instrument.Lakeshore.Lakeshore336
    
    Lakeshore336.autotune_command('GPIB0::15::INSTR',2,1)

    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    output = str(output)
    mode = str(mode)

    request = 'ATUNE ' + output + ',' + mode

    ################## FUNCTION ###############################################

    instru.write(request)

    command_remote_interface_mode(address, 0)

    return


# %%
def command_heater_setup(address, output, heater_resistance, max_current, max_user_current, current_or_power):
    """
    ========== DESCRIPTION ==========

    This function can setup the heater parameters

    ========== FROM ==========

    Manual of Lakeshore 336 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')
        
    <output>
        -- int --
        The channel output (1 = output 1,
                            2 = output 2)
        []
        
    <heater_resistance>
        -- int --
        The heater resistance (1 = 25 Ohms,
                               2 = 50 Ohms)
        []

    <max_current>
        -- int --
        The maximum current (0 = user specified
                             1 = 0.707 A,
                             2 = 1 A,
                             3 = 1.141 A,
                             4 = 2 A)
        []        

    <max_user_current>
        -- int --
        The maxumum current setup by user
        [A]        
        
    <current_or_power>
        -- int --
        Is output displayed as current or power(1 = current,
                                                2 = power)
        []        
          
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========


    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    output = str(output)
    heater_resistance = str(heater_resistance)
    max_current = str(max_current)
    max_user_current = str(max_user_current)
    current_or_power = str(current_or_power)

    request = 'HTRSET ' + output + ',' + heater_resistance + ',' + max_current + ',' + max_user_current + ',' + current_or_power

    ################## FUNCTION ###############################################

    instru.write(request)

    command_remote_interface_mode(address, 0)

    return


# %%
def query_heater_setup(address, output):
    """
    ========== DESCRIPTION ==========

    This function can ask the heater parameters

    ========== FROM ==========

    Manual of Lakeshore 336 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')
        
    <output>
        -- int --
        The channel output (1 = output 1,
                            2 = output 2)
        []
        
    ========== OUTPUT ==========
    
    <heater_resistance>
        -- int --
        The heater resistance (1 = 25 Ohms,
                               2 = 50 Ohms)
        []

    <max_current>
        -- int --
        The maximum current (0 = user specified
                             1 = 0.707 A,
                             2 = 1 A,
                             3 = 1.141 A,
                             4 = 2 A)
        []        

    <max_user_current>
        -- float --
        The maxumum current setup by user
        [A]        
        
    <current_or_power>
        -- int --
        Is output displayed as current or power(1 = current,
                                                2 = power)
        []        
          
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========


    """

    ################## MODULES ################################################

    import pyvisa

    instru = pyvisa.ResourceManager().open_resource(address)

    output = str(output)
    answer = instru.query('HTRSET? ' + output)

    ################## FUNCTION ###############################################

    heater_resistance = int(answer[0:1])
    max_current = int(answer[2:3])
    max_user_current = float(answer[5:10])
    current_or_power = int(answer[11:12])

    command_remote_interface_mode(address, 0)

    return heater_resistance, max_current, max_user_current, current_or_power


# %%
def command_setpoint(address, output, value):
    """
    ========== DESCRIPTION ==========

    This function can setup the setpoint of a specific output

    ========== FROM ==========

    Manual of Lakeshore 336 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')
        
    <output>
        -- int --
        The channel output (1 = output 1,
                            2 = output 2)
        []
        
    <value>
        -- float --
        The setpoint temperature 
        [K]
        
          
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========


    """

    ################## MODULES ################################################

    import pyvisa

    instru = pyvisa.ResourceManager().open_resource(address)

    output = str(output)
    value = str(value)

    ################## FUNCTION ###############################################

    instru.write('SETP ' + output + ',' + value)

    command_remote_interface_mode(address, 0)

    return


# %%
def query_setpoint(address, output):
    """
    ========== DESCRIPTION ==========

    This function can ask the setpoint of a specific output

    ========== FROM ==========

    Manual of Lakeshore 336 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')
        
    <output>
        -- int --
        The channel output (1 = output 1,
                            2 = output 2)
        []
          
    ========== OUTPUT ==========
    
    <setpoint>
        -- float --
        The setpoint temperature 
        [K]
    
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========


    """

    ################## MODULES ################################################

    import pyvisa

    instru = pyvisa.ResourceManager().open_resource(address)

    output = str(output)
    answer = instru.query('SETP? ' + output)

    ################## FUNCTION ###############################################

    setpoint = float(answer)

    command_remote_interface_mode(address, 0)

    return setpoint


# %%
def command_heater_range(address, output, value):
    """
    ========== DESCRIPTION ==========

    This function can setup the range of a specific heater

    ========== FROM ==========

    Manual of Lakeshore 336 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')
        
    <output>
        -- int --
        The channel output (1 = output 1,
                            2 = output 2)
        []
        
    <value>
        -- int --
        The wanted range (0 = Off,
                          1 = Low,
                          2 = Medium,
                          3 = High)
        []
        
          
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========


    """

    ################## MODULES ################################################

    import pyvisa

    instru = pyvisa.ResourceManager().open_resource(address)

    output = str(output)
    value = str(value)

    ################## FUNCTION ###############################################

    instru.write('RANGE ' + output + ',' + value)

    command_remote_interface_mode(address, 0)

    return


# %%
def query_heater_range(address, output):
    """
    ========== DESCRIPTION ==========

    This function can ask the heater range

    ========== FROM ==========

    Manual of Lakeshore 336 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')
        
    <output>
        -- int --
        The channel output (1 = output 1,
                            2 = output 2)
        []
          
    ========== OUTPUT ==========
    
    <value>
        -- int --
        The wanted range (0 = Off,
                          1 = Low,
                          2 = Medium,
                          3 = High)
        []
        
    
    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========


    """

    ################## MODULES ################################################

    import pyvisa

    instru = pyvisa.ResourceManager().open_resource(address)

    output = str(output)
    answer = instru.query('RANGE? ' + output)

    ################## FUNCTION ###############################################

    value = int(answer)

    command_remote_interface_mode(address, 0)

    return value


# %%
def command_outmode(address, output, mode, channel, powerup):
    """
    ========== DESCRIPTION ==========

    This function can setup the output mode

    ========== FROM ==========

    Manual of Lakeshore 336 on https://www.lakeshore.com/

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')
        
    <output>
        -- int --
        The channel output (1 = output 1,
                            2 = output 2,
                            3 = output 3,
                            4 = output 4)
        []   
    
    <mode>
        -- int --
        The output mode (0 = Off,
                         1 = Closed-loop,
                         2 = Zone,
                         3 = Open loop,
                         4 = Monitor out,
                         5 = Warmup summply)
        []
    
    <channel>
        -- int --
        The channel (0 = None,
                     1 = Channel A,
                     2 = Channel B,
                     3 = Channel C,
                     4 = Channel D)
        []        
    
    <powerup>
        -- int --
        The powerup status (0 = Powerup enable off,
                            1 = Powerup enable on)
        []        

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    import cryopy
    cryopy.Instrument.Lakeshore336.command_outmode('GPIB0::15::INSTR',1,1,1,0)

    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    output = str(output)
    mode = str(mode)
    channel = str(channel)
    powerup = str(powerup)

    answer = 'OUTMODE ' + output + ',' + mode + ',' + channel + ',' + powerup

    ################## FUNCTION ###############################################

    instru.write(answer)

    command_remote_interface_mode(address, 0)

    return



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


# %%
def command_local(address):
    """
    ========== DESCRIPTION ==========

    This function can define the "local" mode of the instrument

    ========== FROM ==========

    Manual of QL355TP

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    instruction = "LOCAL"

    ################## FUNCTION ###############################################

    instru.write(instruction)
    return


# %%
def command_voltage(address, channel, voltage):
    """
    ========== DESCRIPTION ==========

    This function can define a voltage output on a specific channel

    ========== FROM ==========

    Manual of QL355TP

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- int --
        The channel of the instrument from 1 to 2

    <voltage>
        -- float --
        The voltage output
        [V]

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    voltage = str(voltage)
    channel = str(channel)

    instruction = 'V' + channel + ' ' + voltage

    ################## FUNCTION ###############################################

    instru.write(instruction)
    command_local(address)

    return


# %%
def command_voltage_with_checking(address, channel, voltage):
    """
    ========== DESCRIPTION ==========

    This function can define a voltage output on a specific channel with a 
    checking of the voltage output

    ========== FROM ==========

    Manual of QL355TP

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- int --
        The channel of the instrument from 1 to 2

    <voltage>
        -- float --
        The voltage output
        [V]

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    voltage = str(voltage)
    channel = str(channel)

    instruction = 'V' + channel + 'V ' + voltage

    ################## FUNCTION ###############################################

    instru.write(instruction)
    command_local(address)

    return


# %%
def command_ovp(address, channel, voltage):
    """
    ========== DESCRIPTION ==========

    This function can define the over voltage power on a specific channel

    ========== FROM ==========

    Manual of QL355TP

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- int --
        The channel of the instrument from 1 to 2

    <voltage>
        -- float --
        The voltage output
        [V]

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    voltage = str(voltage)
    channel = str(channel)

    instruction = 'OVP' + channel + ' ' + voltage

    ################## FUNCTION ###############################################

    instru.write(instruction)
    command_local(address)

    return


# %%
def command_current(address, channel, current):
    """
    ========== DESCRIPTION ==========

    This function can define the limit current on a specific channel

    ========== FROM ==========

    Manual of QL355TP

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument
        
    <channel>
        -- int --
        The channel of the instrument from 1 to 2
        
    <current>
        -- float --
        The limit current output
        [A]

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    current = str(current)
    channel = str(channel)

    instruction = 'I' + channel + ' ' + current

    ################## FUNCTION ###############################################

    instru.write(instruction)
    command_local(address)

    return


# %%
def command_ocp(address, channel, current):
    """
    ========== DESCRIPTION ==========

    This function can define the over current power on a specific channel

    ========== FROM ==========

    Manual of QL355TP

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- int --
        The channel of the instrument from 1 to 2

    <current>
        -- float --
        The current output
        [A]

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    current = str(current)
    channel = str(channel)

    instruction = 'OCP' + channel + ' ' + current

    ################## FUNCTION ###############################################

    instru.write(instruction)
    command_local(address)

    return


# %%
def query_voltage(address, channel):
    """
    ========== DESCRIPTION ==========

    This function can query a voltage output of a specific channel

    ========== FROM ==========

    Manual of QL355TP

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- int --
        The channel of the instrument from 1 to 2
     
    ========== OUTPUT ==========
    
    <voltage>
        -- float --
        The voltage output
        [V]

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    channel = str(channel)

    instruction = 'V' + channel + '?'
    answer = instru.query('instruction')

    voltage = float(answer[3:8])

    command_local(address)

    ################## FUNCTION ###############################################

    return voltage


# %%
def query_current(address, channel):
    """
    ========== DESCRIPTION ==========

    This function can query a current output of a specific channel

    ========== FROM ==========

    Manual of QL355TP

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- int --
        The channel of the instrument from 1 to 2
     
    ========== OUTPUT ==========
    
    <current>
        -- float --
        The current output
        [A]

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    channel = str(channel)

    instruction = 'I' + channel + '?'
    answer = instru.query('instruction')

    current = float(answer[3:9])

    command_local(address)

    ################## FUNCTION ###############################################

    return current


# %%
def command_range(address, channel, range_value):
    """
    ========== DESCRIPTION ==========

    This function can define the range of a specific output

    ========== FROM ==========

    Manual of QL355TP

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- int --
        The channel of the instrument from 1 to 2
        
    <range_value>
        -- int --
        The range value (0  =   15 V (5 A)
                         1  =   35 V (3 A)
                         2  =   35 V (500 mA))

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    range_value = str(range_value)
    channel = str(channel)

    instruction = 'RANGE' + channel + ' ' + range_value

    ################## FUNCTION ###############################################

    instru.write(instruction)
    command_local(address)

    return


# %%
def command_status(address, channel, status):
    """
    ========== DESCRIPTION ==========

    This function can define the status of a specific channel

    ========== FROM ==========

    Manual of QL355TP

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument 
        
    <channel>
        -- int --
        The channel of the instrument from 1 to 2
        
    <status>
        -- int --
        The status (0   =   OFF
                    1   =   ON)

    ========== STATUS ==========

    Status : Checked

    ========= EXAMPLE ==========
    
    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    instru = pyvisa.ResourceManager().open_resource(address)

    status = str(status)
    channel = str(channel)

    instruction = 'OP' + channel + ' ' + status

    ################## FUNCTION ###############################################

    instru.write(instruction)
    command_local(address)

    return



# -*- coding: utf-8 -*-
"""
Created on Fri Aug 12 23:30:35 2022

@author: Administrateur
"""


def query_current(address):
    """
    ========== DESCRIPTION ==========

    This function get the value of the DCI from Keithley 2000

    ========== FROM ==========

    Manual of Keithley 2000

    ========== INPUT ==========

    <address>
        -- string --
        The address of the instrument (e.g. 'GPIB0::15::INSTR')

    ========== OUTPUT ==========

    <current>
        -- float --
        The measured current
        [A]

    ========== STATUS ==========

    Status : Checked


    """

    ################## MODULES ################################################

    import pyvisa

    ################## INITIALISATION #########################################

    rm = pyvisa.ResourceManager()
    instru = rm.open_resource(address)
    instru.write(':CONF:CURR:DC')

    ################## FUNCTION ###############################################

    answer = instru.query(':READ?')
    answer = float(answer)

    return answer
